@@ -1,6 +1,8 @@
 from rx.core.typing import Observable
-
+from PIL import Image
+import numpy as np
 from abc import ABC, abstractmethod
+from typing import Coroutine
 
 from typing import (
     Dict,
@@ -19,7 +21,10 @@ from typing import (
     AsyncGenerator,
 )
 
-Callback = Callable[..., Any]
+Callback = Union[
+    Callable[..., Any],
+    Callable[..., Coroutine[Any, Any, Any]],
+]
 
 
 # database types
@@ -128,8 +133,15 @@ class BaseState(TypedDict):
     lastTriggered: int
 
 
+class Detection(TypedDict):
+    id: Optional[str]
+    label: Optional[str]
+    boundingBox: Tuple[int, int, int, int]
+
+
 class MotionSetEvent(TypedDict):
     state: bool
+    detections: List[Detection]
 
 
 class AudioSetEvent(TypedDict):
@@ -138,7 +150,7 @@ class AudioSetEvent(TypedDict):
 
 class ObjectSetEvent(TypedDict):
     state: bool
-    detections: List[str]
+    detections: List[Detection]
 
 
 class LightSetEvent(TypedDict):
@@ -396,6 +408,7 @@ PluginJsonSchema = Union[
 
 PluginJsonSchemaForm = Dict[str, PluginJsonSchema]
 
+
 class PluginRootSchema(TypedDict):
     schema: PluginJsonSchemaForm
 
@@ -624,6 +637,41 @@ class PluginsManager(Protocol):  # todo: add methods
     def remove_all_listeners(self, event: str) -> "PluginsManager": ...
 
 
+class ImageCrop(TypedDict):
+    top: float
+    left: float
+    bottom: float
+    right: float
+
+
+class ImageResize(TypedDict, total=False):
+    width: Optional[int]
+    height: Optional[int]
+    filter: str
+
+
+class ImageOptions(TypedDict, total=False):
+    format: str
+    crop: ImageCrop
+    resize: ImageResize
+    rotate: int
+
+
+@runtime_checkable
+class VideoFrame(Protocol):
+    metadata: FrameMetadata
+    image: Image.Image
+
+    def input_width(self) -> int: ...
+    def input_height(self) -> int: ...
+    def input_format(self) -> str: ...
+    def to_buffer(self, options: Optional[ImageOptions] = None) -> bytes: ...
+    def to_numpy(
+        self, options: Optional[ImageOptions] = None
+    ) -> np.ndarray[Any, Any]: ...
+    def to_image(self, options: Optional[ImageOptions] = None) -> Image.Image: ...
+
+
 @runtime_checkable
 class CameraDevice(Protocol):
     id: str
@@ -667,7 +715,7 @@ class CameraDevice(Protocol):
     async def disconnect(self) -> None: ...
     async def reboot(self) -> None: ...
     async def snapshot(self, force_new: Optional[bool] = None) -> bytes: ...
-    async def generate_frames(self) -> AsyncGenerator[Frame, None]: ...
+    async def generate_frames(self) -> AsyncGenerator[VideoFrame, None]: ...
     async def get_ffmpeg_path(self) -> str: ...
     async def get_ice_servers(self) -> List[IceServer]: ...
     # async def create_session(self, options: Any) -> Any: ... # todo
@@ -748,11 +796,25 @@ DeviceManagerEventType = Literal[
     "cameraUpdated",
 ]
 
-CameraSelectedCallback = Callable[[CameraDevice, CameraExtension], None]
-CameraDeselectedCallback = Callable[[str, CameraExtension], None]
-CameraAddedCallback = Callable[[CameraDevice], None]
-CameraRemovedCallback = Callable[[str], None]
-CameraUpdatedCallback = Callable[[str, CameraDevice], None]
+CameraSelectedCallback = Union[
+    Callable[[CameraDevice, CameraExtension], None],
+    Callable[[CameraDevice, CameraExtension], Coroutine[None, None, None]],
+]
+CameraDeselectedCallback = Union[
+    Callable[[str, CameraExtension], None],
+    Callable[[str, CameraExtension], Coroutine[None, None, None]],
+]
+CameraAddedCallback = Union[
+    Callable[[CameraDevice], None],
+    Callable[[CameraDevice], Coroutine[None, None, None]],
+]
+CameraRemovedCallback = Union[
+    Callable[[str], None], Callable[[str], Coroutine[None, None, None]]
+]
+CameraUpdatedCallback = Union[
+    Callable[[str, CameraDevice], None],
+    Callable[[str, CameraDevice], Coroutine[None, None, None]],
+]
 
 
 @runtime_checkable
@@ -834,9 +896,16 @@ class PluginAPI(Protocol):
 # base plugin
 class BasePlugin(ABC):
     @abstractmethod
-    async def on_form_submit(self, action_id: str, payload: Any) -> Optional[Dict[Any, Any]]:
+    def __init__(self, logger: PluginLogger, api_instance: PluginAPI) -> None: ...
+
+    @abstractmethod
+    def on_form_submit(
+        self, action_id: str, payload: Any
+    ) -> Optional[Dict[Any, Any]] | Coroutine[Any, Any, Optional[Dict[Any, Any]]]:
         pass
 
     @abstractmethod
-    def configure_cameras(self, cameras: List[CameraDevice]) -> None:
+    def configure_cameras(
+        self, cameras: List[CameraDevice]
+    ) -> None | Coroutine[Any, Any, None]:
         pass
